from web_engine.engine.abstract.AbstractParser import AbstractParser

from interface import implements
from lxml import html, etree
import requests
from urllib.parse import urljoin


class LxmlParser(AbstractParser):
    pass

